import requests
from pandas import DataFrame
from bs4 import BeautifulSoup

def tyo_ipos(instance):
    response = requests.get('http://www.jpx.co.jp/english/listing/stocks/new/index.html')
    soup = BeautifulSoup(response.content,"html.parser")
    for sup in soup.find_all('sup'):
        sup.unwrap()
    s1 = soup.find_all("div", {"class": "component-normal-table"})
    s2 = s1[0].find_all("table", {"class": "fix-header"})
    s3 = s2[0].find_all("tr")
    s4 = s3[0].find_all("th")

    test_data = []
    for i in range(len(s4)):
        test_data.append(s4[i].text)
    tyo_dataframe = DataFrame(columns=test_data)
    s5 = s2[0].find_all("tbody")
    s6 = s5[0].find_all("tr")
    pos = 0
    for tr in s5[0].find_all("tr"):
        temp_list = []
        for td in tr.find_all("td"):
            temp_str = str(td.text)
            temp_list.append(temp_str.strip())
        tyo_dataframe.loc[pos] = temp_list
        pos += 1

    return tyo_dataframe



from time import sleep
from bs4 import BeautifulSoup
from pandas import DataFrame

def nse_ipos(instance):
    instance.get('https://www.nseindia.com/products/content/equities/ipos/homepage_ipo.htm')
    sleep(5)
    source = instance.page_source

    nse_soup = BeautifulSoup(source)

    nse_div = nse_soup.find_all('div', attrs={"class": "tabular-data-historic"})

    sleep(5)
    nse_tbody = nse_div[0].find_all('tbody')

    head = nse_tbody[0].find_all('tr', attrs={'class': 'alt'})
    table_headers = []
    for data in head[0].find_all('td'):
        table_headers.append(data.string)

    table_headers.append('Symbol')
    #     table_headers.append('Issue Type')

    nse_dataframe = DataFrame(columns=table_headers)
    tr = nse_tbody[0].find_all('tr')

    pos = 0
    for i in range(1, len(tr)):
        temp_list = []
        td = tr[i].find_all('td')

        for data in td:
            temp_list.append(data.string)

        link = td[0].find_all('a')
        if link:
            link = link[0].get('href')
            instance.get('https://www.nseindia.com' + link)
            sleep(5)
            extra_data = instance.page_source
            extra_soup = BeautifulSoup(extra_data)

            extra_div = extra_soup.find_all("div", attrs={"id": "ipo_mid"})
            extra_table = extra_div[0].find_all('table')

            extra_tbody = extra_table[0].find_all('tbody')
            extra_tr = extra_tbody[0].find_all('tr')

            for j in range(1, len(extra_tr)):
                td = extra_tr[j].find_all('td')
                temp_str = td[0].string
                try:
                    temp_str = temp_str.replace(' ', '')
                except:
                    pass
                if temp_str == 'Symbol':
                    temp_list.append(td[1].string)
                    break

        if len(temp_list) >= 1:
            try:
                nse_dataframe.loc[pos] = temp_list
                pos += 1
            except:
                pass


    return nse_dataframe


from bs4 import BeautifulSoup
from pandas import DataFrame
from time import sleep



def redundant(tmx_soup):
    tmx_table = tmx_soup.find_all('table', attrs={'class': 'sizeable'})

    header = tmx_table[0].find_all('thead')
    tmx_tr = header[0].find_all('tr')
    header_list = []
    for th in tmx_tr[0].find_all('th'):
        header_list.append(th.string)


    tmx_dataframe = DataFrame(columns=header_list)
    tmx_body = tmx_table[0].find_all('tbody')
    pos = 0
    for tr in tmx_body[0].find_all('tr'):
        temp_list = []
        for td in tr.find_all('td'):
            links = td.find('a')
            if links is None:
                temp_str = tr.find('td', attrs={'class' :'nowrap'})
                temp_str = temp_str.text
                temp_list.append(temp_str[4:])
            else:
                temp_list.append(links.text)

        tmx_dataframe.loc[pos] = temp_list
        pos += 1

    return tmx_dataframe


def tsx_ipos(instance):
    instance.get('https://www.tsx.com/listings/listing-with-us/listed-company-directory?GetPage=ListedCompaniesViewPage&ListedCompanyTab=RecentlyListed&Market=V&Language=en')

    instance.find_element_by_id('exchange-toggle').click()
    sleep(10)
    tsx_soup = BeautifulSoup(instance.page_source)
    tsx_dataframe = redundant(tsx_soup)

    return tsx_dataframe


from bs4 import BeautifulSoup
import requests
from pandas import DataFrame

def lse_ipos(instance):
    response = requests.get('http://www.londonstockexchange.com/exchange/prices-and-markets/stocks/new-and-recent-issues/new-issues.html')
    soup = BeautifulSoup(response.content,"html.parser")
    lse_scrape = soup.find_all("table", {"class": "table_dati"})
    lse_scrape = lse_scrape[0]
    all_th = lse_scrape.find_all("th")
    table_list = []
    for i in range(len(all_th) - 1):
        for th in all_th[i].find_all("p", {"class": "floatsx linenormal"}):
            table_list.append(th.string)
    lse_dataframe = DataFrame(columns=table_list)
    lse_dataframe['amount raised'] = ''
    pos = 0
    tbody = lse_scrape.find_all("tbody")
    all_tr = tbody[0].find_all("tr")
    for a in range(len(all_tr)):
        temp_list = []
        for i in all_tr[a].find_all("td"):
            temp_list.append(i.string)
        lse_dataframe.loc[pos] = temp_list
        pos += 1

    return lse_dataframe


from selenium import webdriver
from ipos_list import nyse,nasdaq,lse,euronext,sehk,bse,nsei,tyo,tsx

class get_ipos():
    def __init__(self, path):
        """
        :param path: expects the path of the chrome driver.

        """
        try:
            #self.path = path
            self.headless = webdriver.ChromeOptions()
            self.headless.add_argument('--headless')
            self.headless.add_argument('--disable-gpu')
            self.headless_browser = webdriver.Chrome(path, options=self.headless)
        except Exception as e:
            print(e)

    def nyse(self):
        """
        :return: dataframe, containing all the recent ipos

        """
        return nyse.nyse_ipos(self.headless_browser)

    def nasdaq(self):
        """

        :return: dataframe, containing all the recent ipos
        """
        return nasdaq.nasdaq_ipos(self.headless_browser)


    def tyo(self): #tokoyo stock exchange
        """

        :return: dataframe, containig all the recent issues
        """

        return tyo.tyo_ipos(self.headless_browser)


    def lse(self):
        """

        :return: dataframe, containig all the recent issues
        """
        return lse.lse_ipos(self.headless_browser)

    def euronext(self):
        """
        :return:  dataframe, containig all the recent issues
        """
        return euronext.euronext_ipos(self.headless_browser)


    def sehk(self): #stock exchange of hong kong
        """

        :return: dataframe, containig all the recent issues
        """
        return sehk.sehk_ipos(self.headless_browser)


    def  tsx(self): #toronto stock exchange
        """

        :return: dataframe, containig all the recent issues
        """
        return tsx.tsx_ipos(self.headless_browser)


    def bse(self):
        """

        :return: dataframe, containig all the recent issues
        """

        return bse.bse_ipos(self.headless_browser)



    def nsei(self):
        """

        :return: dataframe, containig all the recent issues
        """

        return nsei.nse_ipos(self.headless_browser)




import requests
from bs4 import BeautifulSoup
import datetime
from pandas import DataFrame


def euronext_ipos(instance):
    response = requests.get('https://www.euronext.com/en/welcome?quicktabs_25=1#quicktabs-25')
    soup = response.content
    euronext_soup = BeautifulSoup(soup,"html.parser")
    euronext_s1 = euronext_soup.find_all("div", {"id": "quicktabs_tabpage_25_1"})
    euronext_s2 = euronext_s1[0].find_all("thead")
    euronext_s3 = euronext_s2[0].find_all("th")
    now = datetime.date.today()
    euronext_temp_list = []
    for i in range(len(euronext_s3)):
        euronext_temp_str = str(euronext_s3[i].text)
        euronext_temp_list.append(euronext_temp_str.strip())
    euronext_df = DataFrame(columns=euronext_temp_list)
    euronext_df['Ticker'] = ''
    euronext_df['Subsector'] = ''
    euronext_df['Issue Type'] = ''
    euronext_s4 = euronext_s1[0].find_all("tbody")
    euronext_s5 = euronext_s4[0].find_all("tr")
    #euronext_s6 = euronext_s5[0].find_all("td")
    euronext_pos = 0
    for euronext_b in range(len(euronext_s5)):
        euronext_final = []
        for euronext_a in euronext_s5[euronext_b].find_all("span", {"class": "date-display-single"}):
            euronext_final.append(euronext_a.text)
        for euronext_a in euronext_s5[euronext_b].find_all("a"):
            euronext_final.append(euronext_a.text)
        for euronext_a in euronext_s5[euronext_b].find_all("td", {
            "class": "views-field views-field-field-iponi-isin-code-value"}):
            euronext_final.append(euronext_a.text)
        for euronext_a in euronext_s5[euronext_b].find_all("td", {"class": "views-field views-field-tid"}):
            euronext_final.append(euronext_a.text)
        for euronext_a in euronext_s5[euronext_b].find_all("td", {"class": "views-field views-field-tid-1"}):
            euronext_final.append(euronext_a.text)
        for euronext_a in euronext_s5[euronext_b].find_all("a"):
            euronext_l1 = euronext_a.get('href')
            euronext_l2 = "https://www.euronext.com" + euronext_l1
            euronext_r1 = requests.get(euronext_l2)
            euronext_soup1 = BeautifulSoup(euronext_r1.content)
            euronext_s10 = euronext_soup1.find_all("div", {"class": "if-block"})
            euronext_s11 = euronext_s10[0].find_all("div")
            l = len(euronext_s11)
            euronext_final.append((euronext_s11[l - 1].find_all('strong'))[0].string)
            euronext_final.append((euronext_s11[l - 3].find_all('strong'))[0].string)
            # Now going for Issue type
            euro_issue = euronext_soup1.find_all("div", {"class": "column right"})
            inside_euro_issue = euro_issue[0].find_all("div", {"class": "if-block"})
            divs_inside = inside_euro_issue[0].find_all("div")
            l = len(divs_inside)

            euronext_final.append(
                (divs_inside[1].find_all('strong'))[0].string)  # If there is a error on this line check the site
        euronext_df.loc[euronext_pos] = euronext_final
        euronext_pos += 1

    return euronext_df



from bs4 import BeautifulSoup
import pandas as pd
import requests

def nasdaq_ipos(instance):
    nasdaq_url = requests.get('http://www.nasdaq.com/markets/ipos/activity.aspx?tab=upcoming')  # opening the url
    nasdaq_soup = BeautifulSoup(nasdaq_url.content, "html.parser")

    header = nasdaq_soup.find_all('div', class_="ipo-pagination-div")  # getting the title of the table
    # the header definig the table contents
    title = header[0].find_all('h4')[0].string

    nasdaq_data = nasdaq_soup.find_all('div', class_="genTable")  # getting the div where the table is stored
    nasdaq_data = nasdaq_data[0].find_all('table')  # now from inside the div taking the table out

    table_headers = nasdaq_data[0].find_all('thead')  # thead stores all the headers

    table_headers_list = []
    for th in table_headers[0].find_all('th'):
        table_headers_list.append(th.string)

    nasdaq_dataframe = pd.DataFrame(columns=table_headers_list)  # creating the dataframe to store the data
    nasdaq_dataframe.name = title

    pos = 0
    nasdaq_data_tbody = nasdaq_data[0].find_all('tbody')  # tbody is where the row data is stored
    for rows in nasdaq_data_tbody[0].find_all('tr'):
        level = 0
        temp_list = []
        cells = rows.find_all('td')  # finding all the td's inside the tr

        while level < len(table_headers_list):  # done this to avoid hardcoding column names
            temp_list.append(cells[level].find(text=True))
            level += 1

        nasdaq_dataframe.loc[pos] = temp_list  # adding the data to the dataframe
        pos += 1

    return nasdaq_dataframe


from bs4 import BeautifulSoup
import pandas as pd


def nyse_ipos(instance):
    instance.get('https://www.nyse.com/ipo-center/recent-ipo')

    nyse_data = instance.page_source

    nyse_soup = BeautifulSoup(nyse_data)
    nyse_soup_data = nyse_soup.find_all('table', attrs={'class':'table-condensed'})
    nyse_soup_data = nyse_soup_data[0]

    table_headers = nyse_soup_data.find_all('thead')

    table_headers_list = []
    for th in table_headers[0].find_all('th'):
        table_headers_list.append(th.string)

    pos = 0
    nyse_dataframe = pd.DataFrame(columns=table_headers_list)

    nyse_ipo_table = nyse_soup_data.find_all('tbody')

    for rows in nyse_ipo_table[0].find_all('tr'):
        level = 0
        temp_list = []
        cells = rows.find_all('td')  # finding all the td's inside the tr

        while level < len(table_headers_list):  # done this to avoid hardcoding column names
            temp_list.append(cells[level].find(text=True))
            level += 1

        nyse_dataframe.loc[pos] = temp_list  # adding the data to the dataframe
        pos += 1

    return nyse_dataframe



from bs4 import BeautifulSoup
from pandas import DataFrame
import requests

def sehk_ipos(instance):
    response = requests.get("http://www.hkexnews.hk/reports/newlisting/new_listing_announcements.htm")
    soup = BeautifulSoup(response.content,"html.parser")
    s1 = soup.find_all("table", {"class": "table_grey_border ms-rteTable-BlueTable_ENG"})
    s2 = s1[0].find_all('tbody')
    tr = s2[0].find_all('tr')
    header_list = []
    for td in tr[0].find_all('td'):
        header_list.append(td.string)
    header_list.append('Link1')
    header_list.append('Link2')
    sehk_dataframe = DataFrame(columns=header_list)
    pos = 0
    # counter = 0
    for i in range(1, len(tr)):
        temp_list = []
        link_list = []
        for td in tr[i].find_all('td'):
            #         temp_list.append(td.string)
            temp_str = td.find('a')
            if temp_str != None:
                link_list.append('http://www.hkexnews.hk' + temp_str.get('href'))
            temp_list.append(td.string)

        temp_list.extend(link_list)

        try:
            sehk_dataframe.loc[pos] = temp_list
        except:
            temp_list.append(None)
            sehk_dataframe.loc[pos] = temp_list

        pos += 1

    return sehk_dataframe


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="get_ipos",
    version="0.0.1",
    author="Sarthak Negi",
    author_email="sarthaknegi609@gmail.com",
    description="Get the IPOs listings from top exchanges",
    long_description_content_type="text/markdown",
    url="https://github.com/sarthaknegi/get_ipos",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    )
)


import requests
from bs4 import BeautifulSoup
from pandas import DataFrame


def bse_ipos(instance):
    bse_page = requests.get('http://www.bseindia.com/markets/PublicIssues/IPOIssues_new.aspx?id=1&Type=p')
    bse_soup = BeautifulSoup(bse_page.content, "lxml")

    bse_data = bse_soup.find_all('table', class_='tablesorter')

    A = []
    B = []
    C = []
    D = []
    E = []
    F = []
    G = []

    tbody = bse_data[0].find_all('tbody')

    for rows in tbody[0].find_all('tr'):
        cells = rows.find_all('td')
        A.append(cells[0].find(text=True))
        B.append(cells[1].find(text=True))
        C.append(cells[2].find(text=True))
        D.append(cells[3].find(text=True))
        E.append(cells[4].find(text=True))
        F.append(cells[5].find(text=True))
        G.append(cells[6].find(text=True))

    bse_data = DataFrame()
    bse_data['Security'] = A
    bse_data['Start Date'] = B
    bse_data['End Date'] = C
    bse_data['Offer Price'] = D
    bse_data['Face Value'] = E
    bse_data['Type of Issues'] = F
    bse_data['Issue Status'] = G

    return bse_data

